import { get } from 'lodash';

const __OriginalPage = Page;
/**
 * 支持 connect 的 Page 方法
 * @param pageConfig page 配置及生命周期
 * @param mapStates 需要映射的 redux 属性
 * @param mapDispatch 需要映射的 redux 方法
 * @returns Page
 */
function ConnectPage(pageConfig, mapStates = [], mapDispatch = _ => {}, store) {
  const { subscribe, getState, dispatch } = globalStore || store;
  return __OriginalPage({
    ...pageConfig,
    /**
     * 页面的初始数据
     */
    data: {
      ...pageConfig.data,
    },

    /**
     * 生命周期函数--监听页面加载
     */
    onLoad: function (options) {
      this.handleReduxChange();
      this.mapToDispatch();
      pageConfig.onLoad && pageConfig.onLoad.bind(this)(options);
    },

    getMapStates() {
      const newItems = {};
      const state = getState();
      if (mapStates) {
        if (Array.isArray(mapStates)) {
          mapStates.forEach(key => {
            if (typeof key === 'object') {
              const { property, formatter } = key;
              const value = get(state, property);
              if (value !== undefined && get(this.data, property) !== value) {
                newItems[property] = typeof formatter === 'function' ? formatter(value) : value;
              }
            } else {
              const value = get(state, key);
              if (value !== undefined && get(this.data, key) !== value) {
                newItems[key] = value;
              }
            }
          });
        } else if (typeof mapStates === 'function') {
          const list = mapStates(state) || {};
          Object.keys(list).forEach(key => {
            const value = list[key];
            if (value !== undefined && get(this.data, key) !== value) {
              newItems[key] = value;
            }
          });
        } else if (typeof mapStates === 'object') {
          Object.keys(mapStates).forEach(element => {
            const key = mapStates[element];
            if (typeof key === 'object') {
              const { property, formatter } = key;
              const value = get(state, property || element);
              if (value !== undefined && get(this.data, property || element) !== value) {
                newItems[property || element] =
                  typeof formatter === 'function' ? formatter(value) : value;
              }
            } else {
              const value = get(state, mapStates[element]);
              if (typeof value !== 'undefined' && this.data[element] !== value) {
                newItems[element] = value;
              }
            }
          });
        }
      }
      return newItems;
    },

    handleReduxChange() {
      this.setData({
        ...this.getMapStates(),
      });
    },

    clearStoreSubscribe() {
      if (this.storeSubscribe) {
        this.storeSubscribe();
        this.storeSubscribe = undefined;
      }
    },

    mapToDispatch() {
      if (typeof mapDispatch === 'function') {
        const map = mapDispatch(dispatch);
        if (typeof map === 'object') {
          Object.keys(map).forEach(item => {
            this[item] = map[item];
          });
        }
      }
    },

    /**
     * 生命周期函数--监听页面初次渲染完成
     */
    onReady: function () {
      pageConfig.onReady && pageConfig.onReady.bind(this)(...arguments);
    },

    /**
     * 生命周期函数--监听页面显示
     */
    onShow: function () {
      pageConfig.onShow && pageConfig.onShow.bind(this)(...arguments);
      if (!this.storeSubscribe) {
        this.storeSubscribe = subscribe(() => this.handleReduxChange());
      }

      if (!this._isOnLoad) {
        this.setData({
          ...this.getMapStates(),
        });
      }
    },

    /**
     * 生命周期函数--监听页面隐藏
     */
    onHide: function () {
      pageConfig.onHide && pageConfig.onHide.bind(this)(...arguments);
      this.clearStoreSubscribe();
    },

    /**
     * 生命周期函数--监听页面卸载
     */
    onUnload: function () {
      pageConfig.onUnload && pageConfig.onUnload.bind(this)(...arguments);
      this.clearStoreSubscribe();
    },

    /**
     * 页面相关事件处理函数--监听用户下拉动作
     */
    onPullDownRefresh: function () {
      pageConfig.onPullDownRefresh && pageConfig.onPullDownRefresh.bind(this)(...arguments);
    },

    /**
     * 页面上拉触底事件的处理函数
     */
    onReachBottom: function () {
      pageConfig.onReachBottom && pageConfig.onReachBottom.bind(this)(...arguments);
    },
  });
}
let globalStore = null;
export function setGlobalStore(store) {
    globalStore = store;
}
if(typeof ty === 'object'){
  ty.ConnectPage = ConnectPage;
}

export default ConnectPage;
